@@ -3,17 +3,17 @@ import pandas as pd
 from aislab.gnrl.bf import *
 from aislab.gnrl.sf import *
 from aislab.gnrl.measr import *
+#from aislab.gnrl.measr import confm
+from aislab.dp_feng.binenc import cut
+
 
 # exp1:                                                 # y = a + b*e^x
 # exp2:                                                 # y = a + b*e^(x + c)
 # exp3:   ym = pm[0] + pm[1]*np.exp(x*pm[2])            # y = a + b*e^(x*c)
-
 # log:    ym = pm[0] + pm[1]*np.log(abs(x) + 1)         # ym = a + b*ln(x)
 # log1:   ym = pm[0] + pm[1]*np.log(pm[2]*abs(x) + 1)   # y = a + b*ln(c*|x| + 1)
 # logW:   ym = pm[0] + pm[1]*np.log(abs(pm[2]*x + 1))   # y = a + b*ln(|c*x + 1|)
-
 # lgr:    ym = 1/(1 + np.exp(-x@pm))                    # y = 1/(1 + e^-(x*pm))
-
 # exp model
 # args['b'] = 1e-8
 # args['c'] = 1e3
@@ -21,6 +21,109 @@ from aislab.gnrl.measr import *
 # args['b'] = 0.01
 # args['c'] = 1e3
 
+
+###################################################################################
+def f_cbng(X, args): # F objective
+    # Objective function for cross binning with binary dependent
+    # X - population (xi = X[i, :] - i-th individual
+    # args - all remaining arguments of F
+    # F - objective function
+
+    Cd0 = args['Cd0']       # desired lower my (in cell c[0, 0])
+    Cdn = args['Cdn']       # desired upper my (in cell c[-1, -1])
+    nwd0 = args['nwd0']     # desired weighted number of observations in c[0, 0]
+    nwdn = args['nwdn']     # desired weighted number of observations in c[-1, -1]
+    miny0 = args['miny0']   # minimum acceptable number of observations with y=0 in each cell c[i, j]
+    miny01 = args['miny01'] # minimum number of observations with y={0, 1} in each cell c[i, j]
+    mindC = args['mindC']   # minimum change in the criterion moving from one to a neighbour cell
+    wi = args['wi']         # weights of components of F objective
+    nc1 = args['nc1']       # number of cut-offs w.r.t. row variable
+    nc2 = args['nc2']       # number of cut-offs w.r.t. col variable
+    lb1 = args['lb1']       # lower bound for row cut-offs
+    lb2 = args['lb2']       # lower bound for col cut-offs
+    ub1 = args['ub1']       # upper bound for row cut-offs
+    ub2 = args['ub2']       # upper bound for col cut-offs
+    minx0 = args['minx']
+    maxx0 = args['maxx']
+
+    data = args['data'] # todo: svc
+
+    if X.ndim == 1: X = np.array([X])
+    N = len(X)
+    F = nans((N,))
+    i = 0
+    for x in X:
+        x1, ss = sort(c_(x[:nc1, ]))
+        x2, ss = sort(c_(x[nc1:, ]))
+        minx = np.min([minx0, np.min(x)])  # upper bound for row cut-offs
+        maxx = np.max([maxx0, np.max(x)])  # upper bound for col cut-offs
+        x1 = (x1 - minx)/(maxx - minx)*(ub1 - lb1) + lb1
+        x2 = (x2 - minx)/(maxx - minx)*(ub2 - lb2) + lb2
+        x1 = np.round(x1.flatten())
+        x2 = np.round(x2.flatten())
+
+        if len(set(x1)) != len(x1) or len(set(x2)) != len(x2):
+            F[i] = np.inf
+            i += 1
+            continue # skip individual with 2 or more edqual cut-offs per variable => F = inf
+        try:
+            x1_zones = cut(data[:, 0], x1).flatten()
+            x2_zones = cut(data[:, 1], x2).flatten()
+        except:
+            raise Exception('In f_cbng(): Wrong cut-offs in ' + str(i) + '-th individual.')
+        Y1 = confm(-x1_zones, -x2_zones, w=data[:, 2]).astype(int)  # confusion matrix
+        Y0 = confm(-x1_zones, -x2_zones, w=data[:, 3]).astype(int)  # confusion matrix
+        Yna = confm(-x1_zones, -x2_zones, w=data[:, 4]).astype(int)  # confusion matrix
+        import warnings
+        warnings.filterwarnings("ignore", category=RuntimeWarning)
+        Crit = Y0/(Y1 + Y0)*100
+        T01 = Y1 + Y0
+        Tot = Y1 + Y0 + Yna
+        
+        
+        
+        
+        drC = Crit[:, 1:] - Crit[:, :-1]  # rowwise delta Crit
+        dcC = Crit[1:, :] - Crit[:-1, :]  # colwise delta Crit
+
+        mr, nr = np.shape(drC)
+        mc, nc = np.shape(dcC)
+        drC = new(drC)
+        for k in range(nr-1, 0, -1):
+            for j in range(mr-1, 0, -1):
+                if k-1 >= 0 and not np.isnan(drC[j, k]) and np.isnan(drC[j, k-1]): drC[j, k-1] = drC[j, k]
+                if j-1 >= 0 and not np.isnan(drC[j, k]) and np.isnan(drC[j-1, k]): drC[j-1, k] = drC[j, k]
+        dcC = new(dcC)
+        for k in range(nc-1, 0, -1):
+            for j in range(mc-1, 0, -1):
+                if k-1 >= 0 and not np.isnan(dcC[j, k]) and np.isnan(dcC[j, k-1]): dcC[j, k-1] = dcC[j, k]
+                if j-1 >= 0 and not np.isnan(dcC[j, k]) and np.isnan(dcC[j-1, k]): dcC[j-1, k] = dcC[j, k]
+        negdrC = drC[drC < mindC]  # negative trend or not enough change in rowwise delta Crit
+        negdcC = dcC[dcC < mindC]  # negative trend or not enough change in colwise delta Crit
+
+        Fi = nans((7, 1))
+        # discriminatory power of the strategy  #  min(F1) = 0
+        Fi[0, 0] = (((Cd0 - Crit[0, 0])/max(1e-6, 100 - Cd0))**2 + ((nwd0 - Tot[0, 0])/max(1e-6, nwd0))**2)/2
+        Fi[1, 0] = (((Cdn - Crit[-1, -1])/max(1e-6, Cdn - 0))**2 + ((nwdn - Tot[-1, -1])/max(1e-6, nwdn))**2)/2
+        # monotonicity of Crit  #  min(F2) = 0
+        Fi[2, 0] = ((np.sum(mindC - negdrC) + np.sum(mindC - negdcC))/(nc1*nc2))**2
+        # mininmum number of Bad and Good & Bad applications per segment  #  min(F4) = min(F5) = 0
+        Fi[3, 0] = (np.sum((miny0 - Y0)*(miny0 - Y0 > 0)*(T01 > 0))/max(1e-6, (nc1 + 1)*(nc2 + 1)*miny0))**2
+        Fi[4, 0] = (np.sum((miny01 - T01)*(miny01 - T01 > 0)*(T01 > 0))/max(1e-6, (nc1 + 1)*(nc2 + 1)*miny01))**2
+        # minimum number of NaN cells: min(F6) = 0
+        Fi[5, 0] = (np.sum(np.sum(np.isnan(Crit)))/((nc1 + 1)*(nc2 + 1)))**2
+        # minimum number of records in most populated cell  # min(F7) = 0
+        Fi[6, 0] = np.max(np.max(Tot))/max(1e-6, sum(sum(Tot)))
+        F[i] = r_(wi)@Fi
+
+        if np.isnan(F[i]):
+            F[i] = np.inf
+            if i == len(F):
+                return F
+        i += 1
+
+    return F
+###################################################################################
 def f_exp1(args):
     # F for model: y = a + b*e^x
     h = args['data'].shape[1]
